@@ -14,20 +14,22 @@ const VmsListNavigation = ({ selectedVm, vms, expanded, toggleExpansion, onUpdat
     getVms(vmId)
   }
 
+  const idPrefix = `vmslistnav`
   const toggleExpandButton = (
-    <div className={style['toggle-expand-button']}>
-      <a href='#' onClick={toggleExpansion}>
+    <div className={`${style['toggle-expand-button']} ${expanded ? style['toggle-expand-button-expanded'] : ''}`}>
+      <a href='#' onClick={toggleExpansion} id={`${idPrefix}-button-expand`}>
         <span className={`fa ${expanded ? 'fa-angle-double-left' : 'fa-angle-double-right'}`} />
       </a>
     </div>
   )
 
   const loadMore = () => {
-    if (vms.get('areAllPagesLoaded')) {
+    if (vms.get('notAllPagesLoaded')) {
       onUpdate(vms.get('page') + 1)
     }
   }
 
+  const emptyList = (<div />)
   let list = null
   if (expanded) {
     const sortFunction = (vmA, vmB) => vmA.get('name').localeCompare(vmB.get('name'))
@@ -37,7 +39,7 @@ const VmsListNavigation = ({ selectedVm, vms, expanded, toggleExpansion, onUpdat
           if (vm.get('id') === selectedVm.get('id')) { // is selected
             return (
               <li role='presentation' className={`${style['item']} ${style['item-selected']}`} key={vm.get('id')}>
-                <span className={style['item-text']}>{vm.get('name')}</span>
+                <span className={style['item-text']} id={`${idPrefix}-item-${vm.get('name')}`}>{vm.get('name')}</span>
               </li>
             )
           }
@@ -45,7 +47,7 @@ const VmsListNavigation = ({ selectedVm, vms, expanded, toggleExpansion, onUpdat
           return (
             <li role='presentation' className={style['item']} key={vm.get('id')}>
               <Link to={`/vm/${vm.get('id')}`} className={style['item-link']} onClick={() => { linkClick(vm.get('id')) }}>
-                <span className={style['item-text']}>{vm.get('name')}</span>
+                <span className={style['item-text']} id={`${idPrefix}-item-${vm.get('name')}`}>{vm.get('name')}</span>
               </Link>
             </li>
           )
@@ -55,15 +57,18 @@ const VmsListNavigation = ({ selectedVm, vms, expanded, toggleExpansion, onUpdat
   }
 
   return (
-    <InfiniteScroll
-      loadMore={loadMore}
-      hasMore={vms.get('areAllPagesLoaded')}
-      >
-      <div className={`${style['main-container']} ${expanded ? style['expanded'] : ''}`}>
-        {toggleExpandButton}
-        {list}
+    <div className={`${style['main-container']} ${expanded ? style['expanded'] : ''}`}>
+      {toggleExpandButton}
+      <div className={style['scrolling-viewport']}>
+        <InfiniteScroll
+          loadMore={loadMore}
+          hasMore={vms.get('notAllPagesLoaded')}
+          useWindow={false}
+          >
+          {list || emptyList}
+        </InfiniteScroll>
       </div>
-    </InfiniteScroll>
+    </div>
   )
 }
 VmsListNavigation.propTypes = {
