@@ -95,7 +95,7 @@ OvirtApi = {
    * @param vm - Single entry from oVirt REST /api/vms
    * @returns {} - Internal representation of a VM
    */
-  vmToInternal ({ vm }: VmType): Object {
+  vmToInternal ({ vm, getSubResources }: { vm: Object, getSubResources: boolean }): Object {
     function vCpusCount ({ cpu }: { cpu: Object }): number {
       if (cpu && cpu.topology) {
         const top = cpu.topology
@@ -107,7 +107,7 @@ OvirtApi = {
       return 0
     }
 
-    return {
+    const parsedVm = {
       name: vm['name'],
       description: vm['description'],
       id: vm['id'],
@@ -170,7 +170,30 @@ OvirtApi = {
       pool: {
         id: vm['vm_pool'] ? vm.vm_pool['id'] : undefined,
       },
+      cdrom: {},
+      sessions: [],
+      display: {
+        smartcardEnabled: vm['display']['smartcard_enabled'] ? vm['display']['smartcard_enabled'] === 'true' : false,
+      },
     }
+    if (getSubResources) {
+      if (vm.disk_attachments && vm.disk_attachments.disk_attachment) {
+        for (let i in vm.disk_attachments.disk_attachment) {
+          parsedVm.disks[vm.disk_attachments.disk_attachment[i].id] = this.diskToInternal({ attachment: vm.disk_attachments.disk_attachment[i], disk: vm.disk_attachments.disk_attachment[i].disk })
+        }
+      }
+
+      if (vm.cdroms && vm.cdroms.cdrom) {
+        parsedVm.cdrom = this.CDRomToInternal({ cdrom: vm.cdroms.cdrom[0] })
+      }
+      if (vm.graphics_consoles && vm.graphics_consoles.graphics_console) {
+        parsedVm.consoles = this.consolesToInternal({ consoles: vm.graphics_consoles })
+      }
+      if (vm.sessions && vm.sessions.session) {
+        parsedVm.sessions = this.sessionsToInternal({ sessions: vm.sessions })
+      }
+    }
+    return parsedVm
   },
   // ----
   /**
@@ -366,6 +389,13 @@ OvirtApi = {
     }
   },
 
+  SSHKeyToInternal ({ sshKey }: { sshKey: Object }): Object {
+    return {
+      id: sshKey.id,
+      key: sshKey.content,
+    }
+  },
+
   consolesToInternal ({ consoles }: { consoles: Object }): Array<Object> {
     return consoles['graphics_console'].map((c: Object): Object => {
       return {
@@ -374,24 +404,9 @@ OvirtApi = {
       }
     }).sort((a: Object, b: Object): number => b.protocol.length - a.protocol.length) // Hack: 'VNC' is shorter then 'SPICE'
   },
-  // ----
-  login ({ credentials }: { credentials: { username: string, password: string }}): Promise<Object> {
-    const url = `${AppConfiguration.applicationContext}/sso/oauth/token?grant_type=urn:ovirt:params:oauth:grant-type:http&scope=ovirt-app-api`
-    const user = credentials.username
-    const pwd = credentials.password
 
-    return $.ajax(url, {
-      type: 'GET',
-      headers: {
-        'Accept': 'application/json',
-        'Authorization': 'Basic ' + new Buffer(`${user}:${pwd}`, 'utf8').toString('base64'),
-      },
-    }).then((data: Object): Promise<Object> => Promise.resolve(data))
-      .catch((data: Object): Promise<Object> => {
-        logDebug(`Ajax failed: ${JSON.stringify(data)}`)
-        return Promise.reject(data)
-      })
-  },
+  // ----
+
   getOvirtApiMeta (): Promise<Object> {
     OvirtApi._assertLogin({ methodName: 'checkOvirtApiVersion' })
     const url = `${AppConfiguration.applicationContext}/api/`
@@ -402,14 +417,20 @@ OvirtApi = {
     const url = `${AppConfiguration.applicationContext}/api/vms/${vmId}`
     return OvirtApi._httpGet({ url })
   },
-  getVmsByPage ({ page }: { page: number }): Promise<Object> {
+  getVmsByPage ({ page, additional }: { page: number, additional: Array<string> }): Promise<Object> {
     OvirtApi._assertLogin({ methodName: 'getVmsByPage' })
-    const url = `${AppConfiguration.applicationContext}/api/vms/;max=${AppConfiguration.pageLimit}?search=SORTBY NAME ASC page ${page}`
+    let url = `${AppConfiguration.applicationContext}/api/vms/;max=${AppConfiguration.pageLimit}?search=SORTBY NAME ASC page ${page}`
+    if (additional && additional.length > 0) {
+      url += `&follow=${additional.join(',')}`
+    }
     return OvirtApi._httpGet({ url })
   },
-  getVmsByCount ({ count }: { count: number }): Promise<Object> {
+  getVmsByCount ({ count, additional }: { count: number, additional: Array<string> }): Promise<Object> {
     OvirtApi._assertLogin({ methodName: 'getVmsByCount' })
-    const url = `${AppConfiguration.applicationContext}/api/vms/;max=${count}?search=SORTBY NAME ASC`
+    let url = `${AppConfiguration.applicationContext}/api/vms/;max=${count}?search=SORTBY NAME ASC`
+    if (additional && additional.length > 0) {
+      url += `&follow=${additional.join(',')}`
+    }
     return OvirtApi._httpGet({ url })
   },
   shutdown ({ vmId, force }: { vmId: string, force: boolean }): Promise<Object> {
@@ -525,7 +546,13 @@ OvirtApi = {
   },
   checkFilter (): Promise<Object> {
     OvirtApi._assertLogin({ methodName: 'checkFilter' })
-    return OvirtApi._httpGet({ url: `${AppConfiguration.applicationContext}/api/permissions`, custHeaders: { Filter: false } })
+    return OvirtApi._httpGet({
+      url: `${AppConfiguration.applicationContext}/api/permissions`,
+      custHeaders: {
+        Filter: false,
+        Accept: 'application/json',
+      },
+    })
   },
 
   getPoolsByPage ({ page }: { page: number }): Promise<Object> {
@@ -581,6 +608,102 @@ OvirtApi = {
       },
     })
   },
+  changeIcon ({ vmId, iconBase64, mimeType }: { vmId: string, iconBase64: string, mimeType: string }): Promise<Object> {
+    OvirtApi._assertLogin({ methodName: 'changeIcon' })
+    const input = JSON.stringify({ large_icon: { data: iconBase64, media_type: mimeType } })
+    return OvirtApi._httpPut({
+      url: `${AppConfiguration.applicationContext}/api/vms/${vmId}`,
+      input,
+    })
+  },
+  changeIconById ({ vmId, iconId }: { vmId: string, iconId: string }): Promise<Object> {
+    OvirtApi._assertLogin({ methodName: 'changeIconById' })
+    const input = JSON.stringify({ large_icon: { id: iconId } })
+    return OvirtApi._httpPut({
+      url: `${AppConfiguration.applicationContext}/api/vms/${vmId}`,
+      input,
+    })
+  },
+  saveSSHKey ({ key, userId, sshId }: { key: string, userId: string, sshId: ?string }): Promise<Object> {
+    OvirtApi._assertLogin({ methodName: 'saveSSHKey' })
+    const input = JSON.stringify({ content: key })
+    if (sshId !== undefined && sshId !== null) {
+      return OvirtApi._httpPut({
+        url: `${AppConfiguration.applicationContext}/api/users/${userId}/sshpublickeys/${sshId}`,
+        input,
+      })
+    } else {
+      return OvirtApi._httpPost({
+        url: `${AppConfiguration.applicationContext}/api/users/${userId}/sshpublickeys`,
+        input,
+      })
+    }
+  },
+  getSSHKey ({ userId }: { userId: string }): Promise<Object> {
+    OvirtApi._assertLogin({ methodName: 'getSSHKey' })
+    return OvirtApi._httpGet({ url: `${AppConfiguration.applicationContext}/api/users/${userId}/sshpublickeys` })
+  },
+
+  /**
+   * @return {Promise.<?string>} promise of option value if options exists, promise of null otherwise.
+   *                             If default value is provided, the method never returns rejected promise and the default
+   *                             value is returned in case of missing option or any error.
+   */
+  getOption ({ optionName, version, defaultValue }: {optionName: string, version: string, defaultValue?: string}): Promise<?string> {
+    const rawPromise = getOptionWithoutDefault(optionName, version)
+
+    if (!defaultValue) {
+      return rawPromise
+    }
+
+    return rawPromise
+      .then(result => result === null ? defaultValue : result)
+      .catch(() => defaultValue)
+  },
+}
+
+/**
+ * @typedef {'general' | '4.2' | '4.1' | '4.0'} OptionVersionType
+ */
+/**
+ *
+ * @param {string} optionName
+ * @param {OptionVersionType} version
+ * @return {Promise.<?string>} promise of option value if options exists, promise of null otherwise.
+ */
+function getOptionWithoutDefault (optionName: string, version: string): Promise<?string> {
+  OvirtApi._assertLogin({ methodName: 'getOption' })
+  return OvirtApi._httpGet({
+    url: `${AppConfiguration.applicationContext}/api/options/${optionName}`,
+    custHeaders: {
+      Accept: 'application/json',
+      Filter: true,
+    },
+  })
+    .then(response => {
+      let result
+      try {
+        result = response.values.system_option_value
+          .filter((valueAndVersion) => valueAndVersion.version === version)
+          .map(valueAndVersion => valueAndVersion.value)[0]
+      } catch (error) {
+        if (error instanceof TypeError) {
+          logDebug(`Response to getting option '${optionName}' has unexpected format:`, response)
+        }
+        throw error
+      }
+      if (result === undefined) {
+        logDebug(`Config option '${optionName}' was not found for version '${version}'.`)
+        return null
+      }
+      return result
+    }, error => {
+      if (error.status === 404) {
+        logDebug(`Config option '${optionName}' doesn't exist in any version.`)
+        return null
+      }
+      throw error
+    })
 }
 
 const Api = OvirtApi
